use bevy::prelude::*;
use bevy::math::Vec3Swizzles;
use crate::*;

pub fn rotate_sys(
    mut holding: Local<Option<(Entity, f32)>>, // main_shape we hold and the angle offset 
    mouse: Res<Input<MouseButton>>,
    wnds: Res<Windows>,
    q_cam: Query<(&Camera, &GlobalTransform), With<MainCamera>>,
    p_size: Res<PointSize>,
    mut transforms: Query<&mut Transform>,
    gtransforms: Query<&GlobalTransform>,
    shapes: Query<&ShapeData>,
) {
    if let Some(mouse_pos) = get_mouse_pos(&q_cam, &wnds) {
        if mouse.just_pressed(MouseButton::Left) && holding.is_none() {
            for sd in shapes.iter().filter(|x| x.shape != CreateShape::Circle /* no reason to rotate circles ;) */) {
                for edge in sd.edges.iter() {
                    if let Ok(gt) = gtransforms.get(*edge) {
                        if (mouse_pos - gt.translation.xy()).length_squared() < p_size.0.powi(2) {
                            // We grabbed an entity, we now need an angle offset
                            let center = gtransforms.get(sd.main_shape).unwrap().translation.xy();
                            let angle_offset = (mouse_pos - center).angle_between(Vec2::X);
                            let angle_offset = transforms.get(sd.main_shape).unwrap().rotation.to_euler(EulerRot::XYZ).2 + angle_offset;

                            *holding = Some((sd.main_shape, angle_offset));
                        }
                    }
                }
            }
        }
        else if mouse.just_released(MouseButton::Left) {
            *holding  = None;
        }
        else if let Some((en, ang_offset)) = *holding {
            let center = gtransforms.get(en).unwrap().translation.xy();
            let ang_curr = (mouse_pos - center).angle_between(Vec2::X);

            if let Ok(mut t) = transforms.get_mut(en) {
                t.rotation = Quat::from_rotation_z(ang_offset - ang_curr);
            }
        }
    }
}